unit Sndbank;

interface

type
  TCTFHeader = packed record
    Signature: array[0..3] of char;
    SomeFlags: Cardinal;
    RecordNr, I0C: Integer;
    end;

  TCTFRecordHeader = packed record
    I00,   // 01 01 00 00
    I04,   // FF FF FF FF
    I08, I0C, I10, I14, I18, I1C, // null
    SubRecordNr, 
    I24, I28, I2C, I30, I34, I38, I3C, I40: Integer; // null
    I44: Word;
    I46, // 0x3F80
    I4A, // 0xC842
    I4E, // 0xC842
    I52, // 0x447A
    I56, // 0x3F80
    I5A: Integer;
    I5E: Word;
    I60, // 0x0168
    I64, // 0x0168
    I68: Integer;
    end;

  TCTFSubRecord = record
    I00, 
    I04, // 000A
    CSWIndex,
    I0C, I10, I14, I18: Integer;
    end;

  TCTFRecord = record
    Header: TCTFRecordHeader;
    SubRecords: array of TCTFSubRecord;
    end;

  TCTFNameEntry = record
    Index: Integer;
    Name: string;
    end;

  TCTFFile = record
    Header: TCTFHeader;
    Records: array of TCTFRecord;
    NameEntries: array of TCTFNameEntry;
    TotalSubRecords: Integer;
    end;

//////////////////////////////////////

  TCSWHeader = record
    Signature: array[0..3] of char;
    SomeFlags: Cardinal;
    RecordNr, I0C, I10, I14: Integer;
    end;

  TCSWRecord = record
    I00, // 0x0001
    I04, // 0x0001
    I08, // 0x7D00
    Length,
    I10, I14, // null
    Offset: Integer;
    end;

  TCSWFile = record
    Header: TCSWHeader;
    Records: array of TCSWRecord;
    WaveData: array of string;
    end;

function LoadCTF(FileName: string): TCTFFile;
function LoadCSW(FileName: string): TCSWFile;

procedure SaveWaveData(WaveData, FileName: string);

implementation
uses
  SysUtils;

function LoadCTF(FileName: string): TCTFFile;
var
  F: file;
  I, J, L: Integer;
begin
  Assign(F, FileName);
  Reset(F, 1);
  BlockRead(F, Result.Header, SizeOf(TCTFHeader));
  Result.TotalSubRecords:=0;
  SetLength(Result.Records, Result.Header.RecordNr);
  for I:=0 to Result.Header.RecordNr-1 do
    begin
    //Write(IntToHex(FilePos(F), 2), ' - ');
    BlockRead(F, Result.Records[I].Header, SizeOf(TCTFRecordHeader));
    //WriteLn(Result.Records[I].Header.SubRecordNr);
    SetLength(Result.Records[I].SubRecords, Result.Records[I].Header.SubRecordNr);
    Inc(Result.TotalSubRecords, Result.Records[I].Header.SubRecordNr);
    for J:=0 to Result.Records[I].Header.SubRecordNr-1 do
      BlockRead(F, Result.Records[I].SubRecords[J], SizeOf(TCTFSubRecord));
    end;
  SetLength(Result.NameEntries, Result.Header.RecordNr);
  for I:=0 to Result.Header.RecordNr-1 do
    begin
    BlockRead(F, Result.NameEntries[I].Index, 4);
    BlockRead(F, L, 4);
    SetLength(Result.NameEntries[I].Name, L);
    BlockRead(F, Result.NameEntries[I].Name[1], L);
    end;
end;

function LoadCSW(FileName: string): TCSWFile;
var
  F: file;
  I: Integer;
  BaseOffset: Integer;
begin
  Assign(F, FileName);
  Reset(F, 1);
  BlockRead(F, Result.Header, SizeOf(TCSWHeader));
  SetLength(Result.Records, Result.Header.RecordNr);
  for I:=0 to Result.Header.RecordNr-1 do
    BlockRead(F, Result.Records[I], SizeOf(TCSWRecord));
  BaseOffset:=FilePos(F);
  while BaseOffset mod $20<>0 do
    Inc(BaseOffset);   // I couldn't find the offset to the start of the wave data section
  SetLength(Result.WaveData, Result.Header.RecordNr);
  for I:=0 to Result.Header.RecordNr-1 do
    begin
    Seek(F, BaseOffset+Result.Records[I].Offset);
    SetLength(Result.WaveData[I], Result.Records[I].Length);
    BlockRead(F, Result.WaveData[I][1], Result.Records[I].Length);
    end;
end;

procedure SaveWaveData(WaveData, FileName: string);
var
  F: file;
  S: string;
  L: Integer;
begin
  Assign(F, FileName);
  ReWrite(F, 1);
  S:='RIFF';              
  BlockWrite(F, S[1], Length(S));
  L:=Length(WaveData)+44; 
  BlockWrite(F, L, 4);
  S:='WAVEfmt '+
    #$10#$00#$00#$00#$01#$00#$02#$00#$80#$3E#$00#$00#$00#$FA#$00#$00+
    #$04#$00#$10#$00#$64#$61#$74#$61;
  BlockWrite(F, S[1], Length(S));
  L:=Length(WaveData);
  BlockWrite(F, L, 4);
  BlockWrite(F, WaveData[1], Length(WaveData));
end;

end.